// Ein kleines Applet zum Thema Analoge Gezeitenberechnungsmaschine
// Written & Copyright (C) 2000 by Klaus Rotter (klaus@rotters.de)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// History: 2000-XX-XX Initial Work
//          2003-03-02 Portet to Java 1.1, added working sliderbar  
 
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.Date;

public class ErdeMond extends Applet implements Runnable
{
    // Buttons initialisieren
    Button start, stop;
    Scrollbar sb;

    int freq;
    int angle;
    int step = 1;

    boolean StopFlag;

    // Allgemeine Variablen

    public void init()
    {
        Font font = new Font("SansSerif",Font.PLAIN,12);
        setFont(font);

        // Buttons zeichnen
        stop = new Button("Stopp");
        stop.addActionListener(new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!StopFlag)
                {
                    start.setLabel("Cont");
                    StopFlag = true;
                }
            }
        });

        start   = new Button("Start");
        start.addActionListener(new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (StopFlag)
                {
                    start.setLabel("Start");
                    StopFlag = false;
                }
                else {
                    angle = 0;
                }
                repaint();
            }
        });

        sb = new Scrollbar(Scrollbar.HORIZONTAL, freq, 5 ,1, 15);
        sb.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                step = sb.getValue();    
                repaint();
            }
        });

        setLayout(null);
        add (start);
        add (stop);
        add (sb);

    	start.setBounds (10, 10,50,20);
    	stop.setBounds  (65, 10,50,20);
        sb.setBounds    (120,10,80,20);
        // Initialisierung der Variablen	
    } // end of init()

    public void paint(Graphics g)
    {	
        start();
    } 
    // end of paint()
	   	
    // Thread fr die Animationen
    
    private Thread anim = null;
    private boolean flag = false;
    
    public void start() 
    {
        if (anim == null) 
        {
            anim = new Thread (this);
            anim.start();   
        }            
    }
    
    public void stop() 
    {      
        anim = null;
    }

    private int gsin(int grad, int amp)
    {
        return (int) ((double) amp * Math.sin((grad*Math.PI)/180.0));
    }

    private int gcos(int grad, int amp)
    {
        return (int) ((double) amp * Math.cos((grad*Math.PI)/180.0));
    }

    private void drawCircle (Graphics g, int x, int y, int r)
    {
        g.drawOval (x-r/2, y-r/2, r, r);
    }

    public void run()
    {
        Graphics g = getGraphics();

        final int cx = 150;
        final int cy = 165;

        final int ee = 40;
        final int em = 110;

        final int re = 90;
        final int rm = 30;

        int x1[] = new int[360];
        int y1[] = new int[360];
        int xp1= 10, yp1=10;


        int x2[] = new int[360];
        int y2[] = new int[360];
        int xp2= 0, yp2=40;

        int x3[] = new int[360];
        int y3[] = new int[360];
        int xp3= -30, yp3=-30;

        int xe, ye, xm, ym, i;

        while (true)
        {
            g.clearRect(10,30,300,300);

            xe = cx - gsin(angle,ee);
            ye = cy - gcos(angle,ee);

            xm = cx + gsin(angle,em);
            ym = cy + gcos(angle,em);

            g.setColor(Color.blue);
            drawCircle(g,xe,ye,re);
            drawCircle(g,xm,ym,rm);

            g.drawLine(xe,ye,xm,ym);

            g.setColor(Color.black);
            g.drawLine(xe,ye-(re/2+5),xe,ye+(re/2+5));
            g.drawLine(xe-(re/2+5),ye,xe+(re/2+5),ye);

            g.drawLine(xm+gsin(angle,rm/2),ym+gcos(angle,rm/2),xm-gsin(angle,rm/2),ym-gcos(angle,rm/2));
            g.drawLine(xm+gsin(angle+90,rm/2),ym+gcos(angle+90,rm/2),xm-gsin(angle+90,rm/2),ym-gcos(angle+90,rm/2));

            g.drawString("Erde",xe-10,ye+re/2+15);
            g.drawString("Mond",xm-12,ym+rm/2+15);

            g.clearRect(200,10,100,30);
            g.drawString("Umlauf: "+String.valueOf((int)(27.3*angle/360.0))+"d",210,25);
            //g.drawString("S:"+String.valueOf((int) step)+"U:"+String.valueOf((int)(27.3*angle/360.0))+"d",200,25);


            g.drawLine(cx-3,cy,cx+3,cy);
            g.drawLine(cx,cy-3,cx,cy+3);

            g.setColor(Color.red);

            x1[angle] = xe + xp1;
            y1[angle] = ye + yp1;
            x2[angle] = xe + xp2;
            y2[angle] = ye + yp2;
            x3[angle] = xe + xp3;
            y3[angle] = ye + yp3;

            for (i=0; i<angle; i++)
            {
                g.drawRect(x1[i],y1[i],0,0);
                g.drawRect(x2[i],y2[i],0,0);
                g.drawRect(x3[i],y3[i],0,0);
            }

            if (!StopFlag) {
                angle += step;
                if (angle > 359) angle = 0;
            }
            try { Thread.sleep(100); } catch (InterruptedException e) {};
        }
    }
}
// end of class

