// Ein kleines Applet zum Thema Analoge Gezeitenberechnungsmaschine
// geschrieben 31.07.2000 von Klaus Rotter
// Copyright (C) 2000 by Klaus Rotter (klaus@rotters.de)

// 2007-10-08 Fixed a bug. Made GUI a little bit nicer...

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.Date;

public class Maschine extends Applet implements Runnable
{
	// Farben
	Color bgcol = new Color(250,240,180);

	// Fonts
	Font std_font, big_font;

    // Buttons initialisieren
    Button weiter;
    Button zurueck;
    Button start, stop, setup;
    Scrollbar ampsb[] = new Scrollbar[4];
    Scrollbar freqsb[]= new Scrollbar[4];

    // Allgemeine Variablen
    public double phi[] = {0, 0, 0, 0}; // Drehwinkel der Grossen rder
    public double fphi[]= {0, 0, 0, 0}; // Drehwinkel der Umlenkrder
    public double phase[] = {Math.PI/4, Math.PI/5, Math.PI/6, Math.PI/7};

    int step = 0;
    final int MAXSTEP = 9;
    long time;
    boolean StopFlag = false;

    public int amp[]  = {50, 40, 30, 20};  // amp[] des Sinusgenerators in Prozent 0..100
    public int freq[] = {10, 20, 30, 40};  // Frequenz * 100 (=0,5 Hz)

    public void init()
    {
        std_font = new Font("SansSerif",Font.BOLD,13);
        big_font = new Font("SansSerif",Font.BOLD,14);

        setFont(std_font);
 	
        // Buttons zeichnen
        weiter  = new Button("Nchster Schritt");
        weiter.addActionListener(new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (step < MAXSTEP) {
                    step++;

                    switch (step) {
                        case 1:
                            ampsb[0].setBounds (10,160,120,20);
                            break;

                        case 7:
                            freqsb[0].setBounds(10,95,80,20);
                            ampsb[0].setBounds (10,120,80,20);
                            freqsb[1].setBounds(10,145,80,20);
                            ampsb[1].setBounds (10,170,80,20);
                            setup.setBounds(600,280,40,20);
                            break;

                        case 8:
                            freqsb[2].setBounds(10,195,80,20);
                            ampsb[2].setBounds (10,220,80,20);
                            break;

                        case 9:
                            freqsb[3].setBounds(10,245,80,20);
                            ampsb[3].setBounds (10,270,80,20);
                            break;
                    }
                }
                repaint();
            }
        });

        zurueck = new Button("Schritt Zurck");
        zurueck.addActionListener(new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (step > 0)
                {
                    step--;

                    switch (step) {
                        case 0:
                            ampsb[0].setBounds (0,0,0,0);
                            break;

                        case 6:
                            freqsb[0].setBounds(10,100,120,20);
                            ampsb[0].setBounds (10,160,120,20);
                            ampsb[1].setBounds (0,0,0,0);
                            freqsb[1].setBounds (0,0,0,0);
                            setup.setBounds(0,0,0,0);
                            break;

                        case 7:
                            ampsb[2].setBounds (0,0,0,0);
                            freqsb[2].setBounds (0,0,0,0);
                            break;

                        case 8:
                            ampsb[3].setBounds (0,0,0,0);
                            freqsb[3].setBounds (0,0,0,0);
                            break;
                    }
                }
                repaint();
            }
        });

        stop = new Button("Stopp");
        stop.addActionListener(new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!StopFlag)
                {
                    start.setLabel("Cont");
                    StopFlag = true;
                }
            }
        });

        start   = new Button("Start");
        start.addActionListener(new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (StopFlag)
                {
                    start.setLabel("Start");
                    StopFlag = false;
                    Date date = new Date();
                    time = date.getTime();
                }
                else {
                    switch (step)
                    {
                        case 1:
                            ampsb[0].setBounds (0,0,0,0);
                            break;

                        case 7:
                            ampsb[1].setBounds (0,0,0,0);
                            freqsb[1].setBounds (0,0,0,0);
                             break;

                        case 8:
                            ampsb[2].setBounds (0,0,0,0);
                            freqsb[2].setBounds (0,0,0,0);
                            break;

                        case 9:
                            ampsb[3].setBounds (0,0,0,0);
                            freqsb[3].setBounds (0,0,0,0);
                            break;
                    }
                    step = 0;
                }
                repaint();
            }
        });

        setup= new Button("Set");
        add (setup);
		setup.setBounds(0,0,0,0);
        setup.addActionListener(new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                final Setup s = new Setup();
                Button setzen = new Button("Setzen");
                s.add (setzen);
                setzen.setBounds (120,270,100,20);
                setzen.addActionListener(new ActionListener() {
                    public void actionPerformed( ActionEvent e) {
                        for (int i=0; i<4; i++) {
                            amp[i] = s.getAmp(i);
                            ampsb[i].setValue(amp[i]);
                            freq[i] = s.getFreq(i);
                            freqsb[i].setValue(freq[i]);
                            phi[i] = s.getPha(i);
                        }
                        // s.dispose();
                        repaint();
                    }
                });
            }
        });

        freqsb[0] = new Scrollbar(Scrollbar.HORIZONTAL, freq[0], 1 ,0, 101);
        ampsb[0]  = new Scrollbar(Scrollbar.HORIZONTAL,  amp[0], 1 ,0, 101);
        freqsb[1] = new Scrollbar(Scrollbar.HORIZONTAL, freq[1], 1 ,0, 101);
        ampsb[1]  = new Scrollbar(Scrollbar.HORIZONTAL,  amp[1], 1 ,0, 101);
        freqsb[2] = new Scrollbar(Scrollbar.HORIZONTAL, freq[2], 1 ,0, 101);
        ampsb[2]  = new Scrollbar(Scrollbar.HORIZONTAL,  amp[2], 1 ,0, 101);
        freqsb[3] = new Scrollbar(Scrollbar.HORIZONTAL, freq[3], 1 ,0, 101);
        ampsb[3]  = new Scrollbar(Scrollbar.HORIZONTAL,  amp[3], 1 ,0, 101);

        setLayout(null);
        add (weiter);
        add (zurueck);
        add (start);
        add (stop);
        add (freqsb[0]);
        add (ampsb[0]);
        add (freqsb[1]);
        add (ampsb[1]);
        add (freqsb[2]);
        add (ampsb[2]);
        add (freqsb[3]);
        add (ampsb[3]);

    	start.setBounds  (10, 20, 57,20);
    	stop.setBounds   (73, 20, 57,20);
        zurueck.setBounds (10, 42, 120,20 );
		weiter.setBounds  (10, 64, 120,20 );
        //freqsb[0].setBounds (10,100, 80,20);
        freqsb[0].setBounds (10,100, 120,20);
        ampsb[0].setBounds (0,0,0,0);
        freqsb[1].setBounds (0,0,0,0);
        ampsb[1].setBounds  (0,0,0,0);
        freqsb[2].setBounds (0,0,0,0);
        ampsb[2].setBounds  (0,0,0,0);
        freqsb[3].setBounds (0,0,0,0);
        ampsb[3].setBounds  (0,0,0,0);
	
        // Initialisierung der Variablen	

    } // end of init()

    public void drawpoint (Graphics g, int x, int y, String name, Color color)
    {
        g.setColor (color);
        g.fillOval (x-1, y-1, 3, 3);   
        g.drawString(name,x+5,y);
    }
    
    public void drawcircle (Graphics g, int x, int y, int r, Color color)
    {
        g.setColor (color);
        g.drawOval (x-r,y-r,2*r,2*r);
    }

    public void drawline (Graphics g, int x, int y, int x1, int y1, Color color)
    {
        g.setColor (color);
            
        g.drawLine (x,y,x1,y1);
    }

    public void paint(Graphics g)
    {	
        int dx, dy;
		double f1, f2, r;
		
        // Farbeinstellungen
		//setBackground (Color.white);
		setBackground(bgcol);

        g.setColor(Color.black);

  		g.drawString("Simulation einer analogen Gezeitenberechnungsmaschine / Fourier Synthese (von Klaus Rotter)",10,15);

  		// g.drawString("Simulation einer analogen",440,420);
        // g.drawString("Gezeitenberechnungsmaschine",440,440);
        // g.drawString("Fourier Synthese",440,460);
        // g.drawString("(c) 2000-2007 Klaus Rotter",440,480);
        // g.drawString(String.valueOf(step)+" / "+String.valueOf(MAXSTEP),135,65);

        switch (step) {
            case 0: g.drawString("Es dreht sich ein Rad mit der Winkelgeschwindigkeit w.",10,350);
                    g.drawString("Die Winkelgeschwindigkeit kann mit dem Frequenz-Regler verstellt werden.",10,370);
            break;
            case 1: g.drawString("Auf diesem Rad befindet sich eine Gewindestange, mit deren Hilfe ein Fhrungstift",10,350);
            		g.drawString("verstellt werden kann.",10,370);
                    g.drawString("Der Abstand des Stiftes von der Drehachse kann mit dem Amplituden-Regler verndert werden.",10,390);
            break;
            case 2: g.drawString("Die Hhe des des Fhrungsstiftes entspricht zu jedem Zeitpunkt dem Sinus",10,350);
                    g.drawString("des Winkels, den die Strecke Raddrehpunkt-Fhrungsstift mit der Horizontalen einnimmt.",10,370);
                    g.drawString("Die Amplitude des Sinus entspricht dabei dem Abstand des Stiftes zu der Raddrehachse.",10,390);
            break;
            case 3: g.drawString("Der Fhrungsstift nimmt eine Fhrungsschiene mit.",10,350);
            break;
            case 4: g.drawString("Die Fhrungsschiene ist fest mit einer nur vertikal bewegbaren",10,350);
                    g.drawString("Fhrungsstange verbunden und bertrgt so die kreisfrmige Bewegung",10,370);
                    g.drawString("in eine eindimensionale vertikale Bewegung.",10,390);
            break;
            case 5: g.drawString("Zentriert auf der Fhrunsstange befindet sich eine drehbar gelagerte Umlenkrolle.",10,350);
            break;
            case 6: g.drawString("Durch die Umlenkrolle luft ein Stahlseil, welches mit einem Stift verbunden ist,",40,370);
                    g.drawString("der die Bewegung aufzeichnet.",40,390);

        }

        if (step < 7) {
            g.drawString("Frequenz = "+String.valueOf((double) freq[0]/100)+" Hz",10,135);

            if (step > 0)
                g.drawString("Amplitude = "+String.valueOf(amp[0])+" %",10,195);
        }
        else {
            g.drawString("f1 = "+String.valueOf((double) freq[0]/100)+" Hz",95,110);
            g.drawString("a1 = "+String.valueOf(amp[0])+" %",95,135);
            g.drawString("f2 = "+String.valueOf((double) freq[1]/100)+" Hz",95,160);
            g.drawString("a2 = "+String.valueOf(amp[1])+" %",95,185);
        }
        if (step > 7) {
            g.drawString("f3 = "+String.valueOf((double) freq[2]/100)+" Hz",95,210);
            g.drawString("a3 = "+String.valueOf(amp[2])+" %",95,235);
        }
        if (step > 8) {
            g.drawString("f4 = "+String.valueOf((double) freq[3]/100)+" Hz",95,260);
            g.drawString("a4 = "+String.valueOf(amp[3])+" %",95,285);
        }
        start();
    } 
    // end of paint()
 	
    public void setPixel(int x, int y, Graphics g)
    {
        g.setColor(Color.red);
        g.drawRect(x,y,2,2);
     } // end of setPixel()

    public boolean handleEvent(Event event)
    {
        if (event.target == freqsb[0]) {
            freq[0] = ((Integer) event.arg).intValue();
            repaint();
        }
        else if (event.target == ampsb[0]) {
            amp[0] = ((Integer) event.arg).intValue();
            repaint();
        }
        else if (event.target == freqsb[1]) {
            freq[1] = ((Integer) event.arg).intValue();
            repaint();
        }
        else if (event.target == ampsb[1]) {
            amp[1] = ((Integer) event.arg).intValue();
            repaint();
        }
        else if (event.target == freqsb[2]) {
            freq[2] = ((Integer) event.arg).intValue();
            repaint();
        }
        else if (event.target == ampsb[2]) {
            amp[2] = ((Integer) event.arg).intValue();
            repaint();
        }
        else if (event.target == freqsb[3]) {
            freq[3] = ((Integer) event.arg).intValue();
            repaint();
        }
        else if (event.target == ampsb[3]) {
            amp[3] = ((Integer) event.arg).intValue();
            repaint();
        }
        else {
            // System.out.println("Unbekannt: "+event);
            return super.handleEvent(event);
        }
        return true;
	}
	// end of handleEvent()

    // Thread fr die Animationen
    
    private Thread anim = null;
    private boolean flag = false;
    
    public void start() 
    {
        if (anim == null) 
        {
            Date date = new Date();
            time = date.getTime();

            anim = new Thread (this);
            anim.start();   
        }            
    }
    
    public void stop() 
    {
        if ((anim != null) && anim.isAlive())
            anim.stop();
        
        anim = null;
    }

    private void drawStift(Graphics g, int x, int y)
    {
        int xp[] = {x,x-5,x-20,x-20,x-5};
        int yp[] = {y,y-5,y-5, y+5, y+5};

        g.drawPolygon (xp, yp, 5);
    }

    private int drawgenerator(int cx, int cy, int r, double phi, int amp, double fphi)
    {
        int y;

        Graphics g = this.getGraphics();

        int sinr = (int)(Math.sin(phi)*r);
        int cosr = (int)(Math.cos(phi)*r);
        int cos90r = (int)(Math.cos(phi+Math.PI/2)*r);

        g.clearRect(cx-r-15, 20, 3*r, 300);
        drawcircle(g, cx, cy, r, Color.black);
        g.drawLine(cx+sinr, cy+cosr, cx-sinr, cy-cosr);
        g.drawLine(cx+cosr, cy+cos90r, cx-cosr, cy-cos90r);

        y = cy+cosr*amp/100;

        if (step > 0)
        {
            g.setColor(Color.green);
            int xp = cx+sinr*amp/100;
            g.fillOval(xp-5, y-5, 10, 10);

            if (step == 2) {
                g.copyArea (cx+2*r-16,40,400,180,1,0);
                g.setColor(Color.red);
                g.drawLine(xp,y,cx+2*r-15,y);
            }

            if (step > 2)
            {
                g.setColor(Color.black);
                g.fillRect(cx-r-15,y-10,2*r+30,5);
                g.fillRect(cx-r-15,y+5,2*r+30,5);
                g.fillRect(cx-r-15,y-5,5,10);
                g.fillRect(cx+r+10,y-5,5,10);
            }
            if (step > 3)
            {
                g.setColor(Color.green);
                g.fillOval(cx-13,cy+r+10,10,10);
                g.fillOval(cx-13,cy-r-20,10,10);
                g.fillOval(cx+2,cy+r+10,10,10);
                g.fillOval(cx+2,cy-r-20,10,10);

                g.setColor(Color.black);
                g.fillRect(cx-3,y-2*r-25,5,2*r+15);
                g.fillRect(cx-3,y+5 ,5,2*r+15);
            }
            if (step > 4)
            {
                int sr = r/2;

                int sinsr = (int)(Math.sin(fphi)*sr);
                int cossr = (int)(Math.cos(fphi)*sr);
                int cos90sr = (int)(Math.cos(fphi+Math.PI/2)*sr);

                g.setColor(Color.white);
                g.fillOval (cx-sr,y-sr,2*sr,2*sr);

                g.setColor(Color.red);
                g.drawOval (cx-sr,y-sr,2*sr,2*sr);

                g.drawLine(cx+sinsr, y+cossr, cx-sinsr, y-cossr);
                g.drawLine(cx+cossr, y+cos90sr, cx-cossr, y-cos90sr);
            }
            if (step > 5)
            {
                g.setColor(Color.red);
                g.fillOval(cx-r/2-10,cy+r+85,10,10);
                g.fillOval(cx+r/2,cy+r+85,10,10);

                //if (step == 5)
                //    g.fillOval(cx+r/2-5,cy+r+95,10,10);
                //else
                //    g.fillOval(cx+r/2,cy+r+95,10,10);

                g.setColor(Color.blue);
                g.drawLine(cx-r/2,y,cx-r/2,cy+r+90);
                g.drawLine(cx+r/2,y,cx+r/2,cy+r+90);
                g.drawLine(cx-(2*r)-r/2+5,cy+r+95,cx-r/2-5,cy+r+95);
            }
        }
        return (cosr*amp/100);
    }

    public void run() 
    {
        final int Radius = 35;
        final int CX = 220;
        final int CY = 170;
        final int Abstand = 3*Radius;

        int y[] = {0,0,0,0};
        int yold = -1;

        Graphics g = this.getGraphics();

        while (true)
        {
            y[0] = drawgenerator (CX, CY, Radius, phi[0], amp[0], fphi[0]);

            if (step > 5)
                fphi[0] = (double) 2*y[0]/Radius;

            if (step > 6)
            {
                y[1] = drawgenerator (CX+Abstand, CY, Radius, phi[1], amp[1], fphi[1]);
                fphi[1] = (double) 2*y[1]/Radius;
                fphi[0] += fphi[1];
            }
            if (step > 7)
            {
                y[2] = drawgenerator (CX+2*Abstand, CY, Radius, phi[2], amp[2], fphi[2]);
                fphi[2] = (double) 2*y[2]/Radius;
                fphi[0] += fphi[2];
                fphi[1] += fphi[2];
            }
            if (step > 8)
            {
                y[3] = drawgenerator (CX+3*Abstand, CY, Radius, phi[3], amp[3], fphi[3]);
                fphi[3] = (double) 2*y[3]/Radius;
                fphi[0] += fphi[3];
                fphi[1] += fphi[3];
                fphi[2] += fphi[3];
            }

            if (!StopFlag)
            {
                Date date = new Date();
                long newtime = date.getTime();
                long diff = newtime-time;        // Differenzzeit in Millisekunden
                time = newtime;

                // System.out.println("Date: "+String.valueOf(newtime)+" diff: "+String.valueOf(diff));

                phi[0] += (2*Math.PI) * (((double) freq[0]/100)*((double) diff/1000)); // phi += 1 Grad

                if (step > 5)
                    phi[1] += (2*Math.PI) * (((double) freq[1]/100)*((double) diff/1000)); // phi += 1 Grad

                if (step > 6)
                    phi[2] += (2*Math.PI) * (((double) freq[2]/100)*((double) diff/1000)); // phi += 1 Grad

                if (step > 7)
                    phi[3] += (2*Math.PI) * (((double) freq[3]/100)*((double) diff/1000)); // phi += 1 Grad
            }
            if (step > 5)
            {
                final int sy = 320;
                final int ey = 480;
                final int sx = 30;
                final int ex = 640;

                g.copyArea (sx,sy,ex,ey-sy,1,0);
                g.setColor(bgcol);
                g.fillRect(0,sy-20,sx+1,ey-sy+20);
				// g.clearRect(0,300,31,200);

                g.setColor(Color.gray);
                for (int i=sy; i<=ey+9; i+= (ey-sy)/10)
                    g.fillRect(sx,i,1,1);

                g.setColor(Color.green);
                g.fillRect(ex,(ey+sy)/2,2,2);

                g.setColor(Color.black);

                int yp = (sy+ey)/2+2*(y[0]+y[1]+y[2]+y[3]);

                if (yold == -1) yold = yp;

                g.setColor(Color.blue);
                g.drawLine(20, CY+Radius+95, CX-(2*Radius), CY+Radius+95);

                g.setColor(Color.red);
                g.fillOval(15,CY+Radius+95,10,10);

                //g.clipRect (0,sy,35,ey-sy);
				if (yp > sy) {
					g.setColor(Color.blue);

					g.drawLine(15, yp-5, 15, CY+Radius+100);

                	g.setColor(Color.red);
					g.drawLine (30, yold, 30, yp);

					drawStift  (g, 25, yp);
				}
                //g.clipRect (1,1,639,479);

                yold = yp;
            }
            try { Thread.sleep(100); } catch (InterruptedException e) {};
                // final int r = 60;
        }
    }        
} 
// end of class

class Setup extends Frame
{
    Scrollbar ampsb[] = new Scrollbar[4];
    Scrollbar freqsb[]= new Scrollbar[4];
    Scrollbar phasb[] = new Scrollbar[4];

    // Allgemeine Variablen

    int amp[]  = {50, 50, 50, 50};  // amp[] des Sinusgenerators in Prozent 0..100
    int freq[] = {50, 50, 50, 50};  // Frequenz * 100 (=0,5 Hz)
    int pha[]  = {0, 90, 180, 270};

    public Setup()
    {
        setLayout(null);
        Font font = new Font("SansSerif",Font.PLAIN,12);
        setFont(font);
        setBackground (Color.lightGray);

        setTitle("Einstellen der Parameter");

        setBounds (100,100,510,320);

        Button ende = new Button("Ende");
        add (ende);
        ende.setBounds (10,270,100,20);
        ende.addActionListener(new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                dispose();
            }
        });

        freqsb[0] = new Scrollbar(Scrollbar.VERTICAL, freq[0], 1 ,0, 101);
        ampsb[0]  = new Scrollbar(Scrollbar.VERTICAL,  amp[0], 1 ,0, 101);
        phasb[0]  = new Scrollbar(Scrollbar.VERTICAL,  pha[0], 1 ,0, 360);
        freqsb[1] = new Scrollbar(Scrollbar.VERTICAL, freq[1], 1 ,0, 101);
        ampsb[1]  = new Scrollbar(Scrollbar.VERTICAL,  amp[1], 1 ,0, 101);
        phasb[1]  = new Scrollbar(Scrollbar.VERTICAL,  pha[1], 1 ,0, 360);
        freqsb[2] = new Scrollbar(Scrollbar.VERTICAL, freq[2], 1 ,0, 101);
        ampsb[2]  = new Scrollbar(Scrollbar.VERTICAL,  amp[2], 1 ,0, 101);
        phasb[2]  = new Scrollbar(Scrollbar.VERTICAL,  pha[2], 1 ,0, 360);
        freqsb[3] = new Scrollbar(Scrollbar.VERTICAL, freq[3], 1 ,0, 101);
        ampsb[3]  = new Scrollbar(Scrollbar.VERTICAL,  amp[3], 1 ,0, 101);
        phasb[3]  = new Scrollbar(Scrollbar.VERTICAL,  pha[3], 1 ,0, 360);

        add (freqsb[0]);
        add (ampsb[0]);
        add (phasb[0]);

        add (freqsb[1]);
        add (ampsb[1]);
        add (phasb[1]);

        add (freqsb[2]);
        add (ampsb[2]);
        add (phasb[2]);

        add (freqsb[3]);
        add (ampsb[3]);
        add (phasb[3]);

        freqsb[0].setBounds(10,70,20,160);
        ampsb[0].setBounds (50,70,20,160);
        phasb[0].setBounds (90,70,20,160);

        freqsb[1].setBounds(140,70,20,160);
        ampsb[1].setBounds (180,70,20,160);
        phasb[1].setBounds (220,70,20,160);

        freqsb[2].setBounds(270,70,20,160);
        ampsb[2].setBounds (310,70,20,160);
        phasb[2].setBounds (350,70,20,160);

        freqsb[3].setBounds(400,70,20,160);
        ampsb[3].setBounds (440,70,20,160);
        phasb[3].setBounds (480,70,20,160);

        show();
    }

    public int getAmp(int num)
    {
        return (amp[num]);
    }

    public int getFreq(int num)
    {
        return (freq[num]);
    }

    public double getPha(int num)
    {
        return (Math.PI*pha[num]/180.0);
    }


    public boolean handleEvent(Event event)
    {
    if (event.target == freqsb[0]) {
        freq[0] = ((Integer) event.arg).intValue();
        repaint();
    }
    else if (event.target == ampsb[0]) {
        amp[0] = ((Integer) event.arg).intValue();
        repaint();
    }
    else if (event.target == phasb[0]) {
        pha[0] = ((Integer) event.arg).intValue();
        repaint();
    }
		
    else if (event.target == freqsb[1]) {
        freq[1] = ((Integer) event.arg).intValue();
        repaint();
    }
    else if (event.target == ampsb[1]) {
        amp[1] = ((Integer) event.arg).intValue();
        repaint();
    }
    else if (event.target == phasb[1]) {
        pha[1] = ((Integer) event.arg).intValue();
        repaint();
    }
   else if (event.target == freqsb[2]) {
        freq[2] = ((Integer) event.arg).intValue();
        repaint();
    }
    else if (event.target == ampsb[2]) {
        amp[2] = ((Integer) event.arg).intValue();
        repaint();
    }
    else if (event.target == phasb[2]) {
        pha[2] = ((Integer) event.arg).intValue();
        repaint();
    }
   else if (event.target == freqsb[3]) {
        freq[3] = ((Integer) event.arg).intValue();
        repaint();
    }
    else if (event.target == phasb[3]) {
        pha[3] = ((Integer) event.arg).intValue();
        repaint();
    }
   else if (event.target == ampsb[3]) {
        amp[3] = ((Integer) event.arg).intValue();
        repaint();
    }
    else {
        // System.out.println("Unbekannt: "+event);
        return super.handleEvent(event);
    }
    return true;
    }
    // end of handleEvent()

    public void paint(Graphics g)
    {
        g.drawString(String.valueOf((double) freq[0]/100)+"Hz",10,60);
        g.drawString(String.valueOf(amp[0])+"%",50,60);
        g.drawString(String.valueOf(pha[0])+"",90,60);


        g.drawString(String.valueOf((double) freq[1]/100)+"Hz",140,60);
        g.drawString(String.valueOf(amp[1])+"%",180,60);
        g.drawString(String.valueOf(pha[1])+"",220,60);


        g.drawString(String.valueOf((double) freq[2]/100)+"Hz",270,60);
        g.drawString(String.valueOf(amp[2])+"%",310,60);
        g.drawString(String.valueOf(pha[2])+"",350,60);


        g.drawString(String.valueOf((double) freq[3]/100)+"Hz",400,60);
        g.drawString(String.valueOf(amp[3])+"%",440,60);
        g.drawString(String.valueOf(pha[3])+"",480,60);


        g.drawString("Freq   Amp   Pha",10,250);
        g.drawString("Generator 1",15,265);

        g.drawString("Freq   Amp   Pha",140,250);
        g.drawString("Generator 2",145,265);

        g.drawString("Freq   Amp   Pha",270,250);
        g.drawString("Generator 3",275,265);

        g.drawString("Freq   Amp   Pha",400,250);
        g.drawString("Generator 4",405,265);
    }
}
